##-------------------------------------------------------------------
## Modified from reminder by eotinb
## by Ruff
##-------------------------------------------------------------------
## Reorganized to work via CvCustomEventManager
## using Civ4lerts as template.
## CvCustomEventManager & Civ4lerts by Gillmer J. Derge
##-------------------------------------------------------------------

from CvPythonExtensions import *
import CvUtil
import Popup as PyPopup
import PyHelpers

gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo

class ReminderEventManager:

	def __init__(self, eventManager):

		ReminderEvent(eventManager)

		self.reminders = []

		# additions to self.Events
		moreEvents = {
			CvUtil.EventReminderStore  : ('', self.__eventReminderStoreApply,  self.__eventReminderStoreBegin),
			CvUtil.EventReminderRecall : ('', self.__eventReminderRecallApply, self.__eventReminderRecallBegin),
		}
		eventManager.Events.update(moreEvents)

	def __eventReminderStoreBegin(self, argsList):
		popup = PyPopup.PyPopup(CvUtil.EventReminderStore, EventContextTypes.EVENTCONTEXT_SELF)
		popup.setHeaderString("Enter number of turns until reminder goes off and reminder text")
		popup.createSpinBox(0, "", 0, 1, 100, 0)
		popup.createEditBox("", 1)
		popup.addButton("Ok")
		popup.addButton("Cancel")
		popup.launch(False, PopupStates.POPUPSTATE_IMMEDIATE)

	def __eventReminderStoreApply(self, playerID, userData, popupReturn):
		if (popupReturn.getButtonClicked() != 1):
			reminderTurn = popupReturn.getSpinnerWidgetValue(0) + gc.getGame().getGameTurn()
			reminderText = popupReturn.getEditBoxString(1)
			newEntry = (reminderTurn, reminderText)
			self.reminders.append(newEntry)

	def __eventReminderRecallBegin(self, argsList):
		thisTurn = gc.getGame().getGameTurn() + 1

		for i in range(len(self.reminders)):
			if (self.reminders[i][0] == thisTurn):
				message = self.reminders[i][1]
				CyInterface().addMessage(CyGame().getActivePlayer(), True, 10, message, None, 2, None, ColorTypes(8), 0, 0, False, False)

				popup = PyPopup.PyPopup(CvUtil.EventReminderRecall, EventContextTypes.EVENTCONTEXT_SELF)
				popup.setHeaderString("Reminder!")
				popup.setBodyString(message)
				popup.launch()

	def __eventReminderRecallApply(self, playerID, userData, popupReturn):
		message = "eventReminderRecallApply"


class AbstractReminderEvent(object):

	def __init__(self, eventManager, *args, **kwargs):
		super(AbstractReminderEvent, self).__init__(*args, **kwargs)

class ReminderEvent(AbstractReminderEvent):

	def __init__(self, eventManager, *args, **kwargs):
		super(ReminderEvent, self).__init__(eventManager, *args, **kwargs)

		eventManager.addEventHandler("kbdEvent", self.onKbdEvent)
		eventManager.addEventHandler("BeginPlayerTurn", self.onBeginPlayerTurn)

		self.eventMgr = eventManager

	def onKbdEvent(self, argsList):
		eventType,key,mx,my,px,py = argsList
		if ( eventType == self.eventMgr.EventKeyDown ):
			theKey=int(key)
			'Check if ALT + M was hit == show dialog box to set up reminder'
			if (theKey == int(InputTypes.KB_M) and self.eventMgr.bAlt):
				self.eventMgr.beginEvent(CvUtil.EventReminderStore)
				return 1
		return 0

	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList

		if (gc.getPlayer(iPlayer).isHuman()):
			self.eventMgr.beginEvent(CvUtil.EventReminderRecall)
			return 1
