## RuffModControl.py - used to control access to RuffMod ini file.
## modified from HOF MOD V1.61.001
## Hall of Fame Mod Context
## Used to access Mod Ini file
## (registry logic borrowed from DrEJlib's CvPath.py)

import os
import os.path
import _winreg
from types import *
from configobj import ConfigObj
import CvUtil
import RuffModControl
from CvPythonExtensions import *

def __getRegValue(root, subkey, name):
    key = _winreg.OpenKey(root, subkey)
    try:
        value = _winreg.QueryValueEx(key, name)
        return value[0]
    finally:
        key.Close()

def __getInstallDir():
    return __getRegValue(_winreg.HKEY_LOCAL_MACHINE, 
            r"Software\Firaxis Games\Sid Meier's Civilization 4 - Warlords",
            "INSTALLDIR")

def get_AutoLog_Path():
    try: AutoLogPath = RuffModConfigFile['AUTOLOG']['AutoLogPath']
    except: AutoLogPath = userDir + '\AutoLog'
    if (AutoLogPath == 'Default') : AutoLogPath = userDir + '\AutoLog'
    if not os.path.isdir(AutoLogPath) : os.makedirs(AutoLogPath)
    return AutoLogPath

def read_ConfigFile():
    return ConfigObj(FileName)

def write_ConfigFile():
    RuffModConfigFile.write()
    return

myDocuments = __getRegValue(_winreg.HKEY_CURRENT_USER,
            r"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders",
            "Personal")
civ4Dir = os.path.basename(__getInstallDir())
userDir =  os.path.join(os.path.join(myDocuments, "My Games"), civ4Dir)

FileName = userDir + "\\RuffMod_2w.ini"
RuffModConfigFile = read_ConfigFile()

##----------------------------------------------------------------
class RuffModConfig:
    def __init__(self):
        pass
        
## Private Functions
    def __get_keyvalue(self, section, key, default):
        try: keyvalue = RuffModConfigFile[section][key]
        except: keyvalue = default
        return keyvalue

    def __set_keyvalue(self, section, key, keyvalue):
        RuffModConfigFile[section][key] = keyvalue
        return
    
    def __get_boolkeyvalue(self, section, key, default):
        try: keyvalue = RuffModConfigFile[section].as_bool(key)
        except: keyvalue = default
        return keyvalue
    
## Get Functions
    def get_int(self, section, key, default):
        keyvalue = self.__get_keyvalue(section, key, default)
        try: keyvalue = int(keyvalue)
        except: keyvalue = default
        return keyvalue

    def get_str(self, section, key, default):
        keyvalue = self.__get_keyvalue(section, key, default)
        if not isinstance(keyvalue, StringTypes) : keyvalue = default
        return keyvalue

    def get_boolean(self, section, key, default):
        keyvalue = self.__get_boolkeyvalue(section, key, default)
        if keyvalue == False :
            return False
        elif keyvalue == True :
            return True
        else :
            return default
        
    def get_float(self, section, key, default):
        keyvalue = self.__get_keyvalue(section, key, default)
        try: keyvalue = float(keyvalue)
        except: keyvalue = default
        return keyvalue
    
## Set Functions
    def set_int(self, section, key, keyvalue):
        self.__set_keyvalue(section, key, keyvalue)
        return
        
    def set_str(self, section, key, keyvalue):
        self.__set_keyvalue(section, key, keyvalue)
        return

    def set_boolean(self, section, key, keyvalue):
        self.__set_keyvalue(section, key, keyvalue)
        return

    def set_float(self, section, key, keyvalue):
        self.__set_keyvalue(section, key, keyvalue)
        return

